; =========================================================================
;                 CLOCK: EduKit84 Demonstration Program
; =========================================================================
;
; Device: PIC16F84A-04/P, Osc: XT, WDT: Off, Put: On, CP: Off.
; 24-hour clock. Multiplexed LED display demonstration.
; =========================================================================


; NOTE:
; Not all comments, variable and label names are translated to English yet.
; We apologize for any inconvenience.


#include "Def.INC"      ; include source file with definitions

; .........................................................................
        org     0       ; PIC16F84A reset vector
        goto    Start   ; jump to program beginning

; .........................................................................
#include "Int.INC"      ; include file with Interrupt Service Routine

; .........................................................................
#include "Sub.INC"      ; include file with subroutines

; .........................................................................
Start   call    Init            ; initialization routine

Real    movlw   D'200'
        movwf   timer           ; set main timer timeout to 1 second
Real_a  decfsz  timer, f        ; 1 sec is over?
        goto    real_b          ; not yet  => continue
        movlw   .200
        movwf   timer           ; yes => set timer again
        btfsc   stat1, stop     ; clock running? (bit stop=0?)
        goto    real_b          ; no  => continue
        movlw   sec             ; yes => update the time
        movwf   fsr
        movlw   60h
        movwf   maxim           ; selection of seconds
        call    aktual          ; aktualizace vtein
        btfss   status, nula      ; sec = 60? (Z=1?)
        goto    real_b          ; ne  => pokraujeme
        movlw   min
        movwf   fsr             ; ano => vbr minut
        call    aktual          ; aktualizace minut
        btfss   status, nula      ; min = 60? (Z=1?)
        goto    real_b          ; ne  => pokraujeme
        movlw   hrs
        movwf   fsr
        movlw   24h
        movwf   maxim           ; ano => vbr hodin
        call    aktual          ; aktualizace hodin
real_b  call    tlacit          ; voln rutiny na zpracovn tlatek
        btfsc   stat1, test     ; testovac reim? (bit test=1?)
        goto    testr           ; ano => testovac reim
        call    zobraz          ; zobrazen asu
        call    w5ms            ; ekej 5ms
        goto    real_a          ; a pod dokola
;***************************************************************************
; Rutina testuje stav tlatek 6 a 7, kadm stiskem tlatka 6 se rozvt
; postupn jeden ze segment vech slicovek a diod D1 a D8, stisknutm
; tlatka 7 ukonme testovac reim a vrtme se do reimu relnho asu.
; Hodiny se bhem testovn chovaj stejn, jako by testovn nebylo, to je
; bu b nebo stoj podle bitu stop. Pro sprvnou funkci je teba propojku
; P1 propojit C0-AN5. K postupnmu rozsvcovn se pouvaj registry segmt
; a anode. Registr segmt se posl na port B a kadm stiskem tlatka nebo
; prbhem autorepeatu se v nm posouv nula, m se docl postupnho roz-
; svtcovn buzench diod. Registr segmt tedy zabezpeuje buzen katod. Bu-
; zen anod zabepeuje registr anode, kter obsahuje postupn hodnoty:
; 1111, 0111, 1011, 1101, 1110 a cyklicky dle. Peklopen hodnot nastane
; vdy po prbhu nuly celm registrem segmt. Registr anode se pole cel na
; port A a v je-li roven 1111, aktivuje se pin C0. Tm je zabezpeeno buzen
; spolen anode bu jedn slicovky nebo diod D1 a D8. Aby nedochzelo
; k peklpn hodnot, nejsou-li stisknuta dn tlatka a segmt = 0ffh,
; je zaveden bit zmena, kterm se zmna povoluje. Jakmile je jednou zmna
; provedena, bit se shod a nastav se teprve a segmt <> 0ffh.
; Poznmka: port C nen implementovn, tedy aktivace pinu C0 probhne pouze
; jako instrukce. segmt D1 a D8 nebudou tedy svtit, propojme-li propojku
; P1 +5V-AN5, pestanou fungovat tlatka, nebo se pes tyto segmt vnut
; na sbrnici nesprvn rovn. Jedin monost vyzkouen diod je propojit
; propojku P1 AN3-AN5, pak budou tyto segmt svtit souasn s slicovkou
; DISP3.
testr   movlw   b'11111111'
        movwf   porta           ; odpojen anod displeje
        movwf   portb           ; odpojen segment disleje a katod D1 a D8
        movwf   segmt           ; kd do registru segmt
        movlw   b'11101111'
        movwf   anode           ; poten hodnota do registru anode
        bcf     stat1, push     ; tlatko putno, bit shozen
        bcf     stat1, ah       ; nebude autorepeat hodin
        bcf     stat1, am       ; ani autorepeat minut
        bsf     stat1, zmena    ; zmna povolena
        clrf    cnt_a           ; nulovn tae autorepeatu
test_a  decfsz  timer, f        ; uplynula 1s?
        goto    test_b          ; ne  => jdeme dle
        movlw   .200
        movwf   timer           ; ano => znovunastaven tae
        btfsc   stat1, stop     ; hodiny b? (bit stop=0?)
        goto    test_b          ; ne  => jdeme dle
        movlw   sec             ; ano => aktualizujeme as
        movwf   fsr
        movlw   60h
        movwf   maxim           ; vbr vtein
        call    aktual          ; aktualizace vtein
        btfss   status, nula      ; sec = 60? (Z=1?)
        goto    test_b          ; ne  => pokraujeme
        movlw   min
        movwf   fsr             ; ano => vbr minut
        call    aktual          ; aktualizace minut
        btfss   status, nula      ; min = 60? (Z=1?)
        goto    test_b          ; ne  => pokraujeme
        movlw   hrs
        movwf   fsr
        movlw   24h
        movwf   maxim           ; ano => vbr hodin
        call    aktual          ; aktualizace hodin
test_b  call    tltest          ; voln rutiny na zpracovn tlatek
        btfss   stat1, test     ; testovac reim? (bit test=1?)
        goto    real_a          ; ne  => reln as beze zmny registru timer
        bsf     porta, 4        ; odpojen anod diod D1 a D8
        movlw   b'11111111'
        movwf   porta           ; odpojen anod displeje
        movf    segmt, w
        movwf   portb           ; kd diod nebo segment na port B
        movlw   0ffh
        xorwf   segmt, w
        btfss   status, nula    ; segmt = 0ffh? (Z=1?)
        goto    test_c          ; ne  => pouijeme stejnou hodnotu
                                ; ano => generace dal hodnoty registru
                                ;        anode v zvislosti na stavu bitu
                                ;        zmna
        btfss   stat1, zmena    ; meme generovat? (bit zmena = 1?)
        goto    test_c          ; ne  => pouijeme stejnou hodnotu
                                ; ano => generujeme dal hodnotu
        bcf     stat1, zmena    ; zmna zakzna
        bsf     status, carry     ; nastaven carry pro rotaci
        rrf     anode, f        ; dal hodnota
        movf    anode, w
        xorlw   b'11111111'
        btfss   status, nula      ; posledn hodnota? (Z=1?)
        goto    test_c          ; ne  => nic se nedje
        movlw   b'11101111'     ; ano => vnucen prvn hodnoty
        movwf   anode
test_c  movf    anode, w
        movwf   porta           ; aktivace spolen anode jedn slicovky
        incf    cnt_m, f        ; inkrementujeme cnt_m
        movlw   b'00000011'
        andwf   cnt_m, f        ; zamaskovn, cnt_m te t jen do 3
        call    w5ms            ; ekej 5ms
        goto    test_a          ; a pod dokola

; .........................................................................
#include "EEPROM.INC"   ; include file for EEPROM contents definition

; .........................................................................
        end             ; end of program, required directive
